//
//  HandleJsonData.swift
//  The purpose of this class is to load and save json data.
//  Fog of Text
//
//  Created by Joe Mccormack on 7/11/20.
//  Copyright © 2020 Joe Mccormack. All rights reserved.
//

import Foundation
import UIKit

/* Container to define accepted JSON data structure; data not accepted if it does not coform to this structure. */
struct ResponseData: Codable {
    let name: String
    let pages: [Pages]
}
struct Pages: Codable {
    let pg: Int
    let paragraphs: [Paragraphs]
}
struct Paragraphs: Codable {
    let p: Int
    let words: [String]
}
/* Object to hold decoded JSON data to use; in referencing "loadedJSONData" is used in place of "pages". */
var loadedJSONData: [Pages]!

class HandleJsonDataClass {
    /* Tracking load state of JSON data */
    var hasLoadedMsg: String = ""
    var hasLoaded: Bool = false
    /* Tracking save state of JSON data */
    var hasSavedMsg: String = ""
    var hasSaved: Bool = false
    
    /* Load jsondefault from app; loaded by default if in-app purchase has not been made.  Once in-app purchase has been made then jsoncustom in app Documents folder is created from the jsondefault and that jsoncustom is used going forward. */
    func loadJSONDefault() -> Bool {
        /* Reset tracking variables. */
        hasLoadedMsg = ""
        hasLoaded = false
        let bundleFilename: String = "jsondefault"
        if let url = Bundle.main.url(forResource: bundleFilename, withExtension: "json") {
            do {
                /* Load Default JSON Data */
                let jsonDataFile = try Data(contentsOf: url)
                let decodedResponse = try JSONDecoder().decode(ResponseData.self, from: jsonDataFile)
                loadedJSONData = decodedResponse.pages
                hasLoadedMsg = "success"
                hasLoaded = true
                /* print(loadedJSONData!) */
            }
            catch {
                hasLoadedMsg = "Default data could not be loaded: \(error.localizedDescription).  The json data may not be in the format expected, preventing the json data from loading correctly."
                hasLoaded = false
            }
        }
        else {
            hasLoadedMsg = "Default data could not be found.  Restarting or re-installing this app may resolve this issue.  If this app is re-installed, any customized data in jsoncustom.json in the app Documents folder will not be preserved."
            hasLoaded = false
        }
        
        return hasLoaded
    }
    
    /* Load jsoncustom from app Documents folder.  This only happens after in-app purchase has been completed. */
    func loadJSONCustom() -> Bool {
        /* Reset tracking variables. */
        hasLoadedMsg = ""
        hasLoaded = false
        /* Get filepath information for the JSON in the app Documents folder */
        let appDocumentsFolder = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: true)
        let fullPath = appDocumentsFolder.appendingPathComponent("jsoncustom").appendingPathExtension("json")
        do {
            let jsonDataFile = try Data(contentsOf: fullPath)
            let decodedResponse = try JSONDecoder().decode(ResponseData.self, from: jsonDataFile)
            loadedJSONData = decodedResponse.pages
            hasLoadedMsg = "success"
            hasLoaded = true
        }
        catch {
            hasLoadedMsg = "Data could not be loaded: \(error.localizedDescription).  There may be a problem with the jsoncustom.json file in the app's Documents folder such as it not being in the format expected.  The jsoncustom.json file can be reset with this app in Settings, but any customized data in that file will not be preserved."
            hasLoaded = false
        }
        
        return hasLoaded
    }
    
    /* Reset jsoncustom in app Documents folder. */
    func reseetJSONCustom() -> Bool {
        /* Reset tracking variables. */
        hasSavedMsg = ""
        hasSaved = false
        var json: Any
        /* Get filepath information for the JSON in the app Documents folder */
        let appDocumentsFolder = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: true)
        let fullPath = appDocumentsFolder.appendingPathComponent("jsoncustom").appendingPathExtension("json")
        /* Get Bundle JSON data to recover from */
        let bundleFilename: String = "jsondefault"
        if let url = Bundle.main.url(forResource: bundleFilename, withExtension: "json") {
            do {
                /* Load Default JSON Data */
                let fileUrl = try Data(contentsOf: url)
                json = try JSONSerialization.jsonObject(with: fileUrl)
                /* Save JSON Data to Documents */
                let data = try JSONSerialization.data(withJSONObject: json, options: [])
                try data.write(to: fullPath, options: [])
                hasSavedMsg = "success"
                hasSaved = true
                /* print(fullPath) */
            }
            catch {
                hasSavedMsg = "Data could not be reset in the app Documents folder: \(error.localizedDescription)"
                hasSaved = false
            }
        }
        else {
            hasSavedMsg = "Source data could not be found in the app.  The jsoncustom.json file in the app Documents folder has not been reset."
            hasSaved = false
        }

        return hasSaved
    }
    
    /* Save JSON from URL specified by the user to jsoncustom.json in app's Documents folder */
    func loadJSONDataFromUrl(targetUrl: String) {
        /* Reset tracking variables. */
        hasLoadedMsg = ""
        hasLoaded = false
        guard let sourceUrl = URL(string: targetUrl) else {
            hasLoadedMsg = "Invalid URL"
            /* Updaate UI */
            loadJSONDataFromUrlHelperUI(msg: hasLoadedMsg, status: hasLoaded)
            return
        }
        let request = NSURLRequest(url: sourceUrl)
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest, completionHandler: { data, response, error -> Void in
            if let data = data {
                /* To avoid errors about "Invalid conversion from throwing function of type '(Data?,URLResponse?,Error?) throws -> Void' to non-throwing function type '(Data?,URLResponse?,Error?) -> Void'", when attempting to manipulate data here, pass data containing the file content to a helper function. */
                /* Validate JSON content before attempting to save it */
                if let decodedResponse = try? JSONDecoder().decode(ResponseData.self, from: data) {
                    self.hasLoadedMsg = "success"
                    self.hasLoaded = true
                    self.loadJSONDataFromUrlHelper(contentData: data.self)
                }
                else {
                    self.hasLoadedMsg = "The URL specified does not appear to have the expected JSON structure."
                    self.hasLoaded = false
                    /* Updaate UI */
                    self.loadJSONDataFromUrlHelperUI(msg: self.hasLoadedMsg, status: self.hasLoaded)
                }
            }
            if error != nil {
                self.hasLoadedMsg = "Data retreival for the URL specified failed: \(error?.localizedDescription ?? "Unknown error")"
                self.hasLoaded = false
                /* Updaate UI */
                self.loadJSONDataFromUrlHelperUI(msg: self.hasLoadedMsg, status: self.hasLoaded)
            }
        })
        task.resume()
        
        /* Updaate UI */
        loadJSONDataFromUrlHelperUI(msg: hasLoadedMsg, status: hasLoaded)

        return
    }
    func loadJSONDataFromUrlHelper(contentData: Data) {
        /* Retrieve filepath information for app's Documents folder on device */
        let appDocumentsFolder = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: true)
        let fullPath = appDocumentsFolder.appendingPathComponent("jsoncustom").appendingPathExtension("json")
        /* For manually opening the file for verification, show the path */
        /* print(fullPath) */
        /* Save the JSON content to the file */
        do {
            /* Build JSON Data Object */
            let json = try JSONSerialization.jsonObject(with: contentData)
            /* Save JSON Data to Documents */
            let data = try JSONSerialization.data(withJSONObject: json, options: [])
            try data.write(to: fullPath, options: [])
            /* Load jsoncustom.json into JSON structure */
            if (loadJSONCustom() == true) {
                hasLoadedMsg = "success"
                hasLoaded = true
            }
            else {
                hasLoadedMsg = "The JSON from the URL was retrieved and saved successfully to jsoncustom.json in the app Documents folder but that file could not be loaded.  More specifically - " + hasLoadedMsg
                hasLoaded = false
            }
        }
        catch {
            hasLoadedMsg = "Data retreival for the URL specified was successful but could not be saved to the app Documents folder: \(error.localizedDescription)"
            hasLoaded = false
        }
    }
    /* Update UI based on results of asynchronous activity */
    func loadJSONDataFromUrlHelperUI(msg: String, status: Bool) {
        /* Not possible to update the user by presenting a trivial alert message for the Second View Controller even if that message resides in a function inside the Second View Controller, being called from within this class.  Leave user guessing. */
    }
    
}

